﻿using Dapper;
using Microsoft.Extensions.Logging;
using Newtonsoft.Json;
using Npgsql;
using ProjectAtelier.Entities;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace ProjectAtelier.Repositories.Implementations;

public class ProductRepository : IProductRepository
{
    private readonly IConnectionString _connectionString;
    private readonly ILogger<ProductRepository> _logger;
    public ProductRepository(IConnectionString connectionString, ILogger<ProductRepository> logger)
    {
        _connectionString = connectionString;
        _logger = logger;
    }
    public void CreateProduct(Product product)
    {
        _logger.LogInformation("Добавление объекта");
        _logger.LogDebug("Объект: {json}", JsonConvert.SerializeObject(product));
        try
        {
            using var connection = new NpgsqlConnection(_connectionString.ConnectionString);
            connection.Open();
            using var transaction = connection.BeginTransaction();
            var queryInsert = @"
                INSERT INTO Products (Name, View, CountMaterial)
                VALUES (@Name, @View, @CountMaterial);
                SELECT MAX(Id) FROM Products";
            var productId = connection.QueryFirst<int>(queryInsert, product, transaction);
            var querySubInsert = @"
                INSERT INTO Product_Materials (ProductId, MaterialId, Count)
                VALUES (@ProductId, @MaterialId, @Count)";
            foreach (var elem in product.ProductMaterial)
            {
                connection.Execute(querySubInsert, new
                {
                    productId,
                    elem.MaterialId,
                    elem.Count
                }, transaction);
            }
            transaction.Commit();
        }
        catch (Exception ex)
        {
            _logger.LogError(ex, "Ошибка при добавлении объекта");
            throw;
        }
    }

    public void DeleteProduct(int id)
    {
        _logger.LogInformation("Удаление объекта");
        _logger.LogDebug("Объект: {id}", id);
        try
        {
            using var connection = new NpgsqlConnection(_connectionString.ConnectionString);
            var queryDelete = @"
                DELETE FROM Products
                WHERE Id=@id";
            connection.Execute(queryDelete, new { id });
        }
        catch (Exception ex)
        {
            _logger.LogError(ex, "Ошибка при удалении объекта");
            throw;
        }
    }

    public IEnumerable<Product> ReadProducts()
    {
        _logger.LogInformation("Получение всех объектов");
        try
        {
            using var connection = new NpgsqlConnection(_connectionString.ConnectionString);
            var querySelect = @"SELECT * FROM Products";
            var product = connection.Query<Product>(querySelect);
            _logger.LogDebug("Полученные объекты: {json}", JsonConvert.SerializeObject(product));
            return product;
        }
        catch (Exception ex)
        {
            _logger.LogError(ex, "Ошибка при чтении объектов");
            throw;
        }
    }

    public Product ReadProductById(int id)
    {
        _logger.LogInformation("Получение объекта по ID: {id}", id);
        try
        {
            using var connection = new NpgsqlConnection(_connectionString.ConnectionString);
            var querySelect = @" 
            SELECT * FROM Products WHERE Id = @Id;
            SELECT m.*, pm.Count FROM Materials m
            JOIN Product_Materials pm ON m.Id = pm.MaterialId
            WHERE pm.ProductId = @Id";
            using var multi = connection.QueryMultiple(querySelect, new { Id = id });
            var product = multi.Read<Product>().FirstOrDefault();
            if (product != null)
            {
                var materials = multi.Read<Material>().ToList();
                product.ProductMaterial = materials.Select(m => new ProductMaterial
                {
                    MaterialId = m.Id,
                    Count = m.UseCount // Исправлено здесь
                }).ToList();
            }
            _logger.LogDebug("Найденный объект: {json}", JsonConvert.SerializeObject(product));
            return product;
        }
        catch (Exception ex)
        {
            _logger.LogError(ex, "Ошибка при чтении объекта по ID: {id}", id);
            throw;
        }
    }

    public void UpdateProduct(Product product)
    {
        _logger.LogInformation("Редактирование объекта");
        _logger.LogDebug("Объект: {json}", JsonConvert.SerializeObject(product));
        try
        {
            using var connection = new NpgsqlConnection(_connectionString.ConnectionString);
            connection.Open();
            using var transaction = connection.BeginTransaction();

            var queryUpdate = @"
            UPDATE Products
            SET Name = @Name, View = @View, CountMaterial = @CountMaterial
            WHERE Id = @Id";
            connection.Execute(queryUpdate, product, transaction);

            var queryDelete = @"
            DELETE FROM Product_Materials
            WHERE ProductId = @ProductId";
            connection.Execute(queryDelete, new { ProductId = product.Id }, transaction);

            var querySubInsert = @"
            INSERT INTO Product_Materials (ProductId, MaterialId, Count)
            VALUES (@ProductId, @MaterialId, @Count)";
            foreach (var elem in product.ProductMaterial)
            {
                connection.Execute(querySubInsert, new
                {
                    ProductId = product.Id,
                    elem.MaterialId,
                    elem.Count
                }, transaction);
            }

            transaction.Commit();
        }
        catch (Exception ex)
        {
            _logger.LogError(ex, "Ошибка при редактировании объекта");
            throw;
        }
    }
}
